@@ -16,10 +16,14 @@ class ProfilePage(EnumModel):
     MEDALS = "medals"
 
 class GameMode(EnumModel):
-    STD    = "osu"
-    TAIKO  = "taiko"
-    CTB    = "fruits"
-    MANIA  = "mania"
+    OSU   = "osu"
+    TAIKO = "taiko"
+    CATCH = "fruits"
+    MANIA = "mania"
+    # deprecated, remove in 3.x.x
+    CTB   = "fruits"
+    # deprecated, remove in 3.x.x
+    STD   = "osu"
 
 class PlayStyles(IntFlagModel):
     MOUSE = 1
@@ -167,9 +171,9 @@ class EventType(EnumModel):
     USER_SUPPORT_GIFT = "userSupportGift"
     USERNAME_CHANGE = "usernameChange"
 
-# TODO this is just a subset of ``RankStatus``, and is only (currently) used for
-# ``EventType.BEATMAPSET_APPROVE``. Find some way to de-duplicate? Could move to
-# ``RankStatus``, but then how to enforce taking on only a subset of values?
+# TODO this is just a subset of `RankStatus`, and is only (currently) used for
+# `EventType.BEATMAPSET_APPROVE`. Find some way to de-duplicate? Could move to
+# `RankStatus`, but then how to enforce taking on only a subset of values?
 class BeatmapsetApproval(EnumModel):
     RANKED = "ranked"
     APPROVED = "approved"
@@ -286,6 +290,66 @@ class BeatmapsetDiscussionVoteSort(EnumModel):
     NEW = "id_desc"
     OLD = "id_asc"
 
+class BeatmapsetSearchCategory(EnumModel):
+    ANY = "any"
+    HAS_LEADERBOARD = "leaderboard"
+    RANKED = "ranked"
+    QUALIFIED = "qualified"
+    LOVED = "loved"
+    FAVOURITES = "favourites"
+    PENDING = "pending"
+    WIP = "wip"
+    GRAVEYARD = "graveyard"
+    MY_MAPS = "mine"
+
+class BeatmapsetSearchMode(EnumModel):
+    # made up value. this is the default option and doesn't cause a value to
+    # appear in the query string.
+    ANY = -1
+    OSU = 0
+    TAIKO = 1
+    CATCH = 2
+    MANIA = 3
+
+class BeatmapsetSearchExplicitContent(EnumModel):
+    HIDE = "hide"
+    SHOW = "show"
+
+class BeatmapsetSearchGenre(EnumModel):
+    # default option, made up value
+    ANY = 0
+    UNSPECIFIED = 1
+    VIDEO_GAME = 2
+    ANIME = 3
+    ROCK = 4
+    POP = 5
+    OTHER = 6
+    NOVELTY = 7
+    HIP_HOP = 9
+    ELECTRONIC = 10
+    METAL = 11
+    CLASSICAL = 12
+    FOLK = 13
+    JAZZ = 14
+
+class BeatmapsetSearchLanguage(EnumModel):
+    # default option, made up value
+    ANY = 0
+    UNSPECIFIED = 1
+    ENGLISH = 2
+    JAPANESE = 3
+    CHINESE = 4
+    KOREAN = 6
+    INSTRUMENTAL = 5
+    FRENCH = 7
+    GERMAN = 8
+    SWEDISH = 9
+    SPANISH = 10
+    ITALIAN = 11
+    RUSSIAN = 12
+    POLISH = 13
+    OTHER = 14
+
 # =================
 # Documented Models
 # =================
@@ -426,7 +490,7 @@ class EventAchivement(Model):
     icon_url: str
     id: int
     name: str
-    # TODO ``grouping`` can probably be enumified (example value: "Dedication"),
+    # TODO `grouping` can probably be enumified (example value: "Dedication"),
     # need to find full list first though
     grouping: str
     ordering: int
