@@ -264,8 +264,8 @@ class BeatmapsetCompact(Model):
     nsfw: bool
     offset: int
     spotlight: bool
-    # documented as being in ``Beatmapset`` only, but returned by
-    # ``api.beatmapset_events`` which uses a ``BeatmapsetCompact``.
+    # documented as being in `Beatmapset` only, but returned by
+    # `api.beatmapset_events` which uses a `BeatmapsetCompact`.
     hype: Optional[Hype]
 
     # optional fields
@@ -492,6 +492,14 @@ class Spotlight(Model):
 class Spotlights(Model):
     spotlights: List[Spotlight]
 
+# return-value wrapper for https://osu.ppy.sh/docs/index.html#get-users.
+class Users(Model):
+    users: List[UserCompact]
+
+# return-value wrapper for https://osu.ppy.sh/docs/index.html#get-beatmaps.
+class Beatmaps(Model):
+    beatmaps: List[Beatmap]
+
 class Rankings(Model):
     beatmapsets: Optional[List[Beatmapset]]
     cursor: CursorT
@@ -803,6 +811,7 @@ class BeatmapDifficultyAttributes(Model):
     overall_difficulty: Optional[float]
     slider_factor: Optional[float]
     speed_difficulty: Optional[float]
+    speed_note_count: Optional[float]
 
     # taiko attributes
     stamina_difficulty: Optional[float]
@@ -922,7 +931,7 @@ class BeatmapsetEvent(Model):
         mapping = {
             BeatmapsetEventType.BEATMAP_OWNER_CHANGE: BeatmapsetEventCommentOwnerChange,
             BeatmapsetEventType.DISCUSSION_DELETE: BeatmapsetEventCommentNoPost,
-            # ``api.beatmapset_events(types=[BeatmapsetEventType.DISCUSSION_LOCK])``
+            # `api.beatmapset_events(types=[BeatmapsetEventType.DISCUSSION_LOCK])`
             # doesn't seem to be recognized, just returns all events. Was this
             # type discontinued?
             # BeatmapsetEventType.DISCUSSION_LOCK: BeatmapsetEventComment,
